"""AUTOGENERATED. DO NOT EDIT."""

from tf_keras.src.regularizers import L1
from tf_keras.src.regularizers import L1 as l1
from tf_keras.src.regularizers import L1L2
from tf_keras.src.regularizers import L2
from tf_keras.src.regularizers import L2 as l2
from tf_keras.src.regularizers import OrthogonalRegularizer
from tf_keras.src.regularizers import OrthogonalRegularizer as orthogonal_regularizer
from tf_keras.src.regularizers import Regularizer
from tf_keras.src.regularizers import deserialize
from tf_keras.src.regularizers import get
from tf_keras.src.regularizers import l1_l2
from tf_keras.src.regularizers import serialize
