"""AUTOGENERATED. DO NOT EDIT."""

from tf_keras.src.applications.vgg16 import VGG16
from tf_keras.src.applications.vgg16 import decode_predictions
from tf_keras.src.applications.vgg16 import preprocess_input
